@@ -29,7 +29,7 @@
 #include <sys/resource.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#ifdef HAVE_SYSCTL
+#ifdef HAVE_SYS_SYSCTL_H
 # include <sys/sysctl.h>
 #endif
 #ifdef HAVE_SYS_STATVFS_H
@@ -64,6 +64,12 @@
 #ifdef HAVE_DL_ITERATE_PHDR
 # include <link.h>
 #endif
+#ifdef HAVE_OPENSSL
+# if defined(HAVE_WOLFSSL)
+#  include <wolfssl/options.h>
+# endif
+# include <openssl/rand.h>
+#endif
 
 #include "sudo_compat.h"
 #include "sudo_digest.h"
@@ -73,6 +79,10 @@
 # define MAP_ANON MAP_ANONYMOUS
 #endif
 
+#ifndef MAP_FAILED
+# define MAP_FAILED ((void *) -1)
+#endif
+
 #define REPEAT 5
 #define min(a, b) (((a) < (b)) ? (a) : (b))
 
@@ -130,6 +140,11 @@ sudo_getentropy(void *buf, size_t len)
 	if (ret != -1)
 		return (ret);
 
+#ifdef HAVE_OPENSSL
+	if (RAND_bytes(buf, len) == 1)
+		return (0);
+#endif
+
 	ret = getentropy_sysctl(buf, len);
 	if (ret != -1)
 		return (ret);
@@ -615,7 +630,7 @@ getentropy_fallback(void *buf, size_t len)
 done:
 	sudo_digest_free(ctx);
 	if (results != NULL)
-		freezero(results, sizeof(results));
+		freezero(results, digest_len);
 	return (ret);
 }
 
