@@ -25,14 +25,27 @@ main(int argc, char *argv[])
 {
 	FILE *fp = stdin;
 	char pattern[1024], string[1024], flagstr[1024];
-	int errors = 0, tests = 0, flags, got, want;
+	int ch, errors = 0, ntests = 0, flags, got, want;
 
 	initprogname(argc > 0 ? argv[0] : "fnm_test");
 
-	if (argc > 1) {
-		if ((fp = fopen(argv[1], "r")) == NULL) {
-			perror(argv[1]);
-			exit(EXIT_FAILURE);
+	while ((ch = getopt(argc, argv, "v")) != -1) {
+	    switch (ch) {
+	    case 'v':
+		/* ignore */
+		break;
+	    default:
+		fprintf(stderr, "usage: %s [-v]\n", getprogname());
+		return EXIT_FAILURE;
+	    }
+	}
+	argc -= optind;
+	argv += optind;
+
+	if (argc > 0) {
+		if ((fp = fopen(argv[0], "r")) == NULL) {
+			perror(argv[0]);
+			return EXIT_FAILURE;
 		}
 	}
 
@@ -66,13 +79,13 @@ main(int argc, char *argv[])
 				    pattern, string, flags, want, got);
 				errors++;
 			}
-			tests++;
+			ntests++;
 		}
 	}
-	if (tests != 0) {
+	if (ntests != 0) {
 		printf("fnmatch: %d test%s run, %d errors, %d%% success rate\n",
-		    tests, tests == 1 ? "" : "s", errors,
-		    (tests - errors) * 100 / tests);
+		    ntests, ntests == 1 ? "" : "s", errors,
+		    (ntests - errors) * 100 / ntests);
 	}
-	exit(errors);
+	return errors;
 }
