@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2019-2021 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2019-2022 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -148,12 +148,14 @@ static struct logsrvd_config {
 	bool compress;
 	bool flush;
 	bool gid_set;
+	bool log_passwords;
 	uid_t uid;
 	gid_t gid;
 	mode_t mode;
 	unsigned int maxseq;
 	char *iolog_dir;
 	char *iolog_file;
+	void *passprompt_regex;
     } iolog;
     struct logsrvd_config_eventlog {
 	int log_type;
@@ -215,6 +217,18 @@ logsrvd_conf_iolog_file(void)
     return logsrvd_config->iolog.iolog_file;
 }
 
+bool
+logsrvd_conf_iolog_log_passwords(void)
+{
+    return logsrvd_config->iolog.log_passwords;
+}
+
+void *
+logsrvd_conf_iolog_passprompt_regex(void)
+{
+    return logsrvd_config->iolog.passprompt_regex;
+}
+
 /* server getters */
 struct server_address_list *
 logsrvd_conf_server_listen_address(void)
@@ -367,6 +381,19 @@ cb_iolog_compress(struct logsrvd_config *config, const char *str, size_t offset)
     debug_return_bool(true);
 }
 
+static bool
+cb_iolog_log_passwords(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    int val;
+    debug_decl(cb_iolog_log_passwords, SUDO_DEBUG_UTIL);
+
+    if ((val = sudo_strtobool(str)) == -1)
+	debug_return_bool(false);
+
+    config->iolog.log_passwords = val;
+    debug_return_bool(true);
+}
+
 static bool
 cb_iolog_flush(struct logsrvd_config *config, const char *str, size_t offset)
 {
@@ -449,6 +476,20 @@ cb_iolog_maxseq(struct logsrvd_config *config, const char *str, size_t offset)
     debug_return_bool(true);
 }
 
+static bool
+cb_iolog_passprompt_regex(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    debug_decl(cb_iolog_passprompt_regex, SUDO_DEBUG_UTIL);
+
+    if (config->iolog.passprompt_regex == NULL) {
+	/* Lazy alloc of the passprompt regex handle. */
+	config->iolog.passprompt_regex = iolog_pwfilt_alloc();
+	if (config->iolog.passprompt_regex == NULL)
+	    debug_return_bool(false);
+    }
+    debug_return_bool(iolog_pwfilt_add(config->iolog.passprompt_regex, str));
+}
+
 /* Server callbacks */
 static bool
 append_address(struct server_address_list *addresses, const char *str,
@@ -538,11 +579,11 @@ cb_server_listen_address(struct logsrvd_config *config, const char *str, size_t
 static bool
 cb_server_timeout(struct logsrvd_config *config, const char *str, size_t offset)
 {
-    int timeout;
-    const char* errstr;
+    time_t timeout;
+    const char *errstr;
     debug_decl(cb_server_timeout, SUDO_DEBUG_UTIL);
 
-    timeout = sudo_strtonum(str, 0, UINT_MAX, &errstr);
+    timeout = sudo_strtonum(str, 0, TIME_T_MAX, &errstr);
     if (errstr != NULL)
 	debug_return_bool(false);
 
@@ -743,11 +784,11 @@ cb_relay_host(struct logsrvd_config *config, const char *str, size_t offset)
 static bool
 cb_relay_timeout(struct logsrvd_config *config, const char *str, size_t offset)
 {
-    int timeout;
-    const char* errstr;
+    time_t timeout;
+    const char *errstr;
     debug_decl(cb_relay_timeout, SUDO_DEBUG_UTIL);
 
-    timeout = sudo_strtonum(str, 0, UINT_MAX, &errstr);
+    timeout = sudo_strtonum(str, 0, TIME_T_MAX, &errstr);
     if (errstr != NULL)
 	debug_return_bool(false);
 
@@ -759,11 +800,11 @@ cb_relay_timeout(struct logsrvd_config *config, const char *str, size_t offset)
 static bool
 cb_relay_connect_timeout(struct logsrvd_config *config, const char *str, size_t offset)
 {
-    int timeout;
-    const char* errstr;
+    time_t timeout;
+    const char *errstr;
     debug_decl(cb_relay_connect_timeout, SUDO_DEBUG_UTIL);
 
-    timeout = sudo_strtonum(str, 0, UINT_MAX, &errstr);
+    timeout = sudo_strtonum(str, 0, TIME_T_MAX, &errstr);
     if (errstr != NULL)
 	debug_return_bool(false);
 
@@ -789,6 +830,22 @@ cb_relay_dir(struct logsrvd_config *config, const char *str, size_t offset)
     debug_return_bool(true);
 }
 
+static bool
+cb_retry_interval(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    time_t interval;
+    const char *errstr;
+    debug_decl(cb_retry_interval, SUDO_DEBUG_UTIL);
+
+    interval = sudo_strtonum(str, 0, TIME_T_MAX, &errstr);
+    if (errstr != NULL)
+	debug_return_bool(false);
+
+    config->relay.retry_interval = interval;
+
+    debug_return_bool(true);
+}
+
 static bool
 cb_relay_store_first(struct logsrvd_config *config, const char *str, size_t offset)
 {
@@ -1046,6 +1103,7 @@ static struct logsrvd_config_entry relay_conf_entries[] = {
     { "timeout", cb_relay_timeout },
     { "connect_timeout", cb_relay_connect_timeout },
     { "relay_dir", cb_relay_dir },
+    { "retry_interval", cb_retry_interval },
     { "store_first", cb_relay_store_first },
     { "tcp_keepalive", cb_relay_keepalive },
 #if defined(HAVE_OPENSSL)
@@ -1069,7 +1127,9 @@ static struct logsrvd_config_entry iolog_conf_entries[] = {
     { "iolog_user", cb_iolog_user },
     { "iolog_group", cb_iolog_group },
     { "iolog_mode", cb_iolog_mode },
+    { "log_passwords", cb_iolog_log_passwords },
     { "maxseq", cb_iolog_maxseq },
+    { "passprompt_regex", cb_iolog_passprompt_regex },
     { NULL }
 };
 
@@ -1126,14 +1186,21 @@ logsrvd_conf_parse(struct logsrvd_config *config, FILE *fp, const char *path)
 
 	/* New section */
 	if (line[0] == '[') {
-	    char *section_name = line + 1;
-	    char *cp = strchr(section_name, ']');
-	    if (cp == NULL) {
+	    char *cp, *section_name = line + 1;
+
+	    if ((ep = strchr(section_name, ']')) == NULL) {
 		sudo_warnx(U_("%s:%d unmatched '[': %s"),
 		    path, lineno, line);
 		goto done;
 	    }
-	    *cp = '\0';
+	    for (cp = ep + 1; *cp != '\0'; cp++) {
+		if (!isspace((unsigned char)*cp)) {
+		    sudo_warnx(U_("%s:%d garbage after ']': %s"),
+			path, lineno, line);
+		    goto done;
+		}
+	    }
+	    *ep = '\0';
 	    for (conf_section = logsrvd_config_sections; conf_section->name != NULL;
 		    conf_section++) {
 		if (strcasecmp(section_name, conf_section->name) == 0)
@@ -1242,7 +1309,7 @@ logsrvd_stub_close_log(int type, FILE *fp)
     return;
 }
 
-/* Set eventlog configuration settings from on logsrvd config. */
+/* Set eventlog configuration settings from logsrvd config. */
 static void
 logsrvd_conf_eventlog_setconf(struct logsrvd_config *config)
 {
@@ -1262,6 +1329,22 @@ logsrvd_conf_eventlog_setconf(struct logsrvd_config *config)
     debug_return;
 }
 
+/* Set I/O log configuration settings from logsrvd config. */
+static void
+logsrvd_conf_iolog_setconf(struct logsrvd_config *config)
+{
+    debug_decl(logsrvd_conf_iolog_setconf, SUDO_DEBUG_UTIL);
+
+    iolog_set_defaults();
+    iolog_set_compress(config->iolog.compress);
+    iolog_set_flush(config->iolog.flush);
+    iolog_set_owner(config->iolog.uid, config->iolog.gid);
+    iolog_set_mode(config->iolog.mode);
+    iolog_set_maxseq(config->iolog.maxseq);
+
+    debug_return;
+}
+
 /*
  * Conversation function for use by sudo_warn/sudo_fatal.
  * Logs to stdout/stderr.
@@ -1483,6 +1566,7 @@ logsrvd_conf_free(struct logsrvd_config *config)
     /* struct logsrvd_config_iolog */
     free(config->iolog.iolog_dir);
     free(config->iolog.iolog_file);
+    iolog_pwfilt_free(config->iolog.passprompt_regex);
 
     /* struct logsrvd_config_logfile */
     free(config->logfile.path);
@@ -1573,6 +1657,7 @@ logsrvd_conf_alloc(void)
     config->iolog.uid = ROOT_UID;
     config->iolog.gid = ROOT_GID;
     config->iolog.gid_set = false;
+    config->iolog.log_passwords = true;
 
     /* Event log defaults */
     config->eventlog.log_type = EVLOG_SYSLOG;
@@ -1619,6 +1704,12 @@ logsrvd_conf_apply(struct logsrvd_config *config)
 #endif
     debug_decl(logsrvd_conf_apply, SUDO_DEBUG_UTIL);
 
+    /* There can be multiple passprompt regular expressions. */
+    if (config->iolog.passprompt_regex == NULL) {
+	if (!cb_iolog_passprompt_regex(config, PASSPROMPT_REGEX, 0))
+	    debug_return_bool(false);
+    }
+
     /* There can be multiple addresses so we can't set a default earlier. */
     if (TAILQ_EMPTY(&config->server.addresses.addrs)) {
 	/* Enable plaintext listender. */
@@ -1738,15 +1829,12 @@ logsrvd_conf_apply(struct logsrvd_config *config)
 	break;
     }
 
-    /* Set I/O log library settings */
-    iolog_set_defaults();
-    iolog_set_compress(config->iolog.compress);
-    iolog_set_flush(config->iolog.flush);
-    iolog_set_owner(config->iolog.uid, config->iolog.gid);
-    iolog_set_mode(config->iolog.mode);
-    iolog_set_maxseq(config->iolog.maxseq);
-
-    /* Set event log config */
+    /*
+     * Update event and I/O log library config and install the new
+     * logsrvd config.  We must not fail past this point or the event
+     * and I/O log config will be inconsistent with the logsrvd config.
+     */
+    logsrvd_conf_iolog_setconf(config);
     logsrvd_conf_eventlog_setconf(config);
 
     logsrvd_conf_free(logsrvd_config);
