@@ -54,7 +54,6 @@
 #include "sudo_iolog.h"
 #include "sudo_util.h"
 
-#include "log_server.pb-c.h"
 #include "logsrvd.h"
 
 /*
@@ -470,6 +469,11 @@ journal_accept(AcceptMessage *msg, uint8_t *buf, size_t len,
 {
     debug_decl(journal_accept, SUDO_DEBUG_UTIL);
 
+    if (closure->journal_path != NULL) {
+	/* Re-use existing journal file. */
+	debug_return_bool(journal_write(buf, len, closure));
+    }
+
     /* Store message in a journal for later relaying. */
     if (!journal_create(closure))
 	debug_return_bool(false);
@@ -500,8 +504,10 @@ journal_reject(RejectMessage *msg, uint8_t *buf, size_t len,
     debug_decl(journal_reject, SUDO_DEBUG_UTIL);
 
     /* Store message in a journal for later relaying. */
-    if (!journal_create(closure))
-	debug_return_bool(false);
+    if (closure->journal_path == NULL) {
+	if (!journal_create(closure))
+	    debug_return_bool(false);
+    }
     if (!journal_write(buf, len, closure))
 	debug_return_bool(false);
 
@@ -535,7 +541,15 @@ journal_alert(AlertMessage *msg, uint8_t *buf, size_t len,
 {
     debug_decl(journal_alert, SUDO_DEBUG_UTIL);
 
-    debug_return_bool(journal_write(buf, len, closure));
+    /* Store message in a journal for later relaying. */
+    if (closure->journal_path == NULL) {
+	if (!journal_create(closure))
+	    debug_return_bool(false);
+    }
+    if (!journal_write(buf, len, closure))
+	debug_return_bool(false);
+
+    debug_return_bool(true);
 }
 
 /*
