@@ -63,14 +63,22 @@ sudo_securid_init(struct passwd *pw, sudo_auth *auth)
     static SDI_HANDLE sd_dat;			/* SecurID handle */
     debug_decl(sudo_securid_init, SUDOERS_DEBUG_AUTH);
 
-    auth->data = (void *) &sd_dat;		/* For method-specific data */
-
-    /* Start communications */
-    if (AceInitialize() != SD_FALSE)
+    /* Only initialize once. */
+    if (auth->data != NULL)
 	debug_return_int(AUTH_SUCCESS);
 
-    sudo_warnx(U_("failed to initialise the ACE API library"));
-    debug_return_int(AUTH_FATAL);
+    if (IS_NONINTERACTIVE(auth))
+        debug_return_int(AUTH_NONINTERACTIVE);
+
+    /* Start communications */
+    if (AceInitialize() == SD_FALSE) {
+	sudo_warnx("%s", U_("failed to initialise the ACE API library"));
+	debug_return_int(AUTH_FATAL);
+    }
+
+    auth->data = (void *) &sd_dat;		/* For method-specific data */
+
+    debug_return_int(AUTH_SUCCESS);
 }
 
 /*
@@ -95,7 +103,7 @@ sudo_securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 
     /* Re-initialize SecurID every time. */
     if (SD_Init(sd) != ACM_OK) {
-	sudo_warnx(U_("unable to contact the SecurID server"));
+	sudo_warnx("%s", U_("unable to contact the SecurID server"));
 	debug_return_int(AUTH_FATAL);
     }
 
@@ -104,23 +112,23 @@ sudo_securid_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 
     switch (retval) {
 	case ACM_OK:
-		sudo_warnx(U_("User ID locked for SecurID Authentication"));
+		sudo_warnx("%s", U_("User ID locked for SecurID Authentication"));
 		debug_return_int(AUTH_SUCCESS);
 
         case ACE_UNDEFINED_USERNAME:
-		sudo_warnx(U_("invalid username length for SecurID"));
+		sudo_warnx("%s", U_("invalid username length for SecurID"));
 		debug_return_int(AUTH_FATAL);
 
 	case ACE_ERR_INVALID_HANDLE:
-		sudo_warnx(U_("invalid Authentication Handle for SecurID"));
+		sudo_warnx("%s", U_("invalid Authentication Handle for SecurID"));
 		debug_return_int(AUTH_FATAL);
 
 	case ACM_ACCESS_DENIED:
-		sudo_warnx(U_("SecurID communication failed"));
+		sudo_warnx("%s", U_("SecurID communication failed"));
 		debug_return_int(AUTH_FATAL);
 
 	default:
-		sudo_warnx(U_("unknown SecurID error"));
+		sudo_warnx("%s", U_("unknown SecurID error"));
 		debug_return_int(AUTH_FATAL);
 	}
 }
@@ -154,17 +162,17 @@ sudo_securid_verify(struct passwd *pw, char *pass, sudo_auth *auth, struct sudo_
 		break;
 
 	case ACE_UNDEFINED_PASSCODE:
-		sudo_warnx(U_("invalid passcode length for SecurID"));
+		sudo_warnx("%s", U_("invalid passcode length for SecurID"));
 		ret = AUTH_FATAL;
 		break;
 
 	case ACE_UNDEFINED_USERNAME:
-		sudo_warnx(U_("invalid username length for SecurID"));
+		sudo_warnx("%s", U_("invalid username length for SecurID"));
 		ret = AUTH_FATAL;
 		break;
 
 	case ACE_ERR_INVALID_HANDLE:
-		sudo_warnx(U_("invalid Authentication Handle for SecurID"));
+		sudo_warnx("%s", U_("invalid Authentication Handle for SecurID"));
 		ret = AUTH_FATAL;
 		break;
 
@@ -176,10 +184,8 @@ sudo_securid_verify(struct passwd *pw, char *pass, sudo_auth *auth, struct sudo_
                 /* Sometimes (when current token close to expire?)
                    ACE challenges for the next token displayed
                    (entered without the PIN) */
-		if (pass != NULL) {
-		    memset_s(pass, SUDO_PASS_MAX, 0, strlen(pass));
-		    free(pass);
-		}
+		if (pass != NULL)
+		    freezero(pass, strlen(pass));
         	pass = auth_getpass("\
 !!! ATTENTION !!!\n\
 Wait for the token code to change, \n\
@@ -209,7 +215,7 @@ then enter the new token code.\n", \
 		break;
 
 	default:
-		sudo_warnx(U_("unknown SecurID error"));
+		sudo_warnx("%s", U_("unknown SecurID error"));
 		ret = AUTH_FATAL;
 		break;
     }
@@ -217,10 +223,8 @@ then enter the new token code.\n", \
     /* Free resources */
     SD_Close(*sd);
 
-    if (pass != NULL) {
-	memset_s(pass, SUDO_PASS_MAX, 0, strlen(pass));
-	free(pass);
-    }
+    if (pass != NULL)
+	freezero(pass, strlen(pass));
 
     /* Return stored state to calling process */
     debug_return_int(ret);
