@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2009-2020 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2009-2022 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -69,8 +69,10 @@ static struct sudoers_io_operations {
 
 static struct log_details iolog_details;
 static bool warned = false;
+static bool log_passwords = true;
 static int iolog_dir_fd = -1;
 static struct timespec last_time;
+static void *passprompt_regex_handle;
 static void sudoers_io_setops(void);
 
 /* sudoers_io is declared at the end of this file. */
@@ -80,7 +82,7 @@ extern sudo_dso_public struct io_plugin sudoers_io;
  * Sudoers callback for maxseq Defaults setting.
  */
 bool
-cb_maxseq(const union sudo_defs_val *sd_un)
+cb_maxseq(const union sudo_defs_val *sd_un, int op)
 {
     const char *errstr;
     unsigned int value;
@@ -104,7 +106,7 @@ cb_maxseq(const union sudo_defs_val *sd_un)
  * Sudoers callback for iolog_user Defaults setting.
  */
 bool
-cb_iolog_user(const union sudo_defs_val *sd_un)
+cb_iolog_user(const union sudo_defs_val *sd_un, int op)
 {
     const char *name = sd_un->str;
     struct passwd *pw;
@@ -129,7 +131,7 @@ cb_iolog_user(const union sudo_defs_val *sd_un)
  * Look up I/O log group-ID from group name.
  */
 bool
-cb_iolog_group(const union sudo_defs_val *sd_un)
+cb_iolog_group(const union sudo_defs_val *sd_un, int op)
 {
     const char *name = sd_un->str;
     struct group *gr;
@@ -154,7 +156,7 @@ cb_iolog_group(const union sudo_defs_val *sd_un)
  * Sudoers callback for iolog_mode Defaults setting.
  */
 bool
-cb_iolog_mode(const union sudo_defs_val *sd_un)
+cb_iolog_mode(const union sudo_defs_val *sd_un, int op)
 {
     iolog_set_mode(sd_un->mode);
     return true;
@@ -232,6 +234,7 @@ deserialize_stringlist(const char *s)
 	    free(str);
 	    goto bad;
 	}
+	unescape_string(str->str);
 	STAILQ_INSERT_TAIL(strlist, str, entries);
     }
     if (STAILQ_EMPTY(strlist))
@@ -244,6 +247,39 @@ bad:
     debug_return_ptr(NULL);
 }
 
+/*
+ * Set passprompt regex filter based on a comma-separated string.
+ * Returns a passprompt regex handle pointer.
+ */
+static void *
+set_passprompt_regex(const char *cstr)
+{
+    void *handle;
+    char *cp, *last, *str;
+    debug_decl(set_passprompt_regex, SUDOERS_DEBUG_UTIL);
+
+    handle = iolog_pwfilt_alloc();
+    str = strdup(cstr);
+    if (handle == NULL || str == NULL) {
+	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	goto bad;
+    }
+
+    for ((cp = strtok_r(str, ",", &last)); cp != NULL;
+	    (cp = strtok_r(NULL, ",", &last))) {
+	unescape_string(cp);
+	if (!iolog_pwfilt_add(handle, cp))
+	    goto bad;
+    }
+
+    free(str);
+    debug_return_ptr(handle);
+bad:
+    free(str);
+    iolog_pwfilt_free(handle);
+    debug_return_ptr(NULL);
+}
+
 /*
  * Pull out I/O log related data from user_info and command_info arrays.
  * Returns true if I/O logging is enabled, false if not and -1 on error.
@@ -441,6 +477,16 @@ iolog_deserialize_info(struct log_details *details, char * const user_info[],
 	    }
 	    break;
 	case 'l':
+	    if (strncmp(*cur, "log_passwords=", sizeof("log_passwords=") - 1) == 0) {
+		int val = sudo_strtobool(*cur + sizeof("log_passwords=") - 1);
+		if (val != -1) {
+		    log_passwords = val;
+		} else {
+		    sudo_debug_printf(SUDO_DEBUG_WARN,
+			"%s: unable to parse %s", __func__, *cur);
+		}
+		continue;
+	    }
 	    if (strncmp(*cur, "log_servers=", sizeof("log_servers=") - 1) == 0) {
 		details->log_servers =
 		    deserialize_stringlist(*cur + sizeof("log_servers=") - 1);
@@ -502,10 +548,19 @@ iolog_deserialize_info(struct log_details *details, char * const user_info[],
 	    if (strncmp(*cur, "maxseq=", sizeof("maxseq=") - 1) == 0) {
 		union sudo_defs_val sd_un;
 		sd_un.str = *cur + sizeof("maxseq=") - 1;
-		cb_maxseq(&sd_un);
+		cb_maxseq(&sd_un, true);
 		continue;
 	    }
 	    break;
+	case 'p':
+	    if (strncmp(*cur, "passprompt_regex=", sizeof("passprompt_regex=") - 1) == 0) {
+		iolog_pwfilt_free(passprompt_regex_handle);
+		passprompt_regex_handle =
+		    set_passprompt_regex(*cur + sizeof("passprompt_regex=") - 1);
+		if (passprompt_regex_handle == NULL)
+		    debug_return_int(-1);
+	    }
+	    break;
 	case 'r':
 	    if (strncmp(*cur, "runas_gid=", sizeof("runas_gid=") - 1) == 0) {
 		runas_gid_str = *cur + sizeof("runas_gid=") - 1;
@@ -852,6 +907,8 @@ sudoers_io_close(int exit_status, int error)
     free_iolog_details();
     sudo_freepwcache();
     sudo_freegrcache();
+    iolog_pwfilt_free(passprompt_regex_handle);
+    passprompt_regex_handle = NULL;
 
     /* sudoers_debug_deregister() calls sudo_debug_exit() for us. */
     sudoers_debug_deregister();
@@ -879,6 +936,7 @@ sudoers_io_log_local(int event, const char *buf, unsigned int len,
 {
     struct iolog_file *iol;
     char tbuf[1024];
+    char *newbuf = NULL;
     int ret = -1;
     debug_decl(sudoers_io_log_local, SUDOERS_DEBUG_PLUGIN);
 
@@ -895,8 +953,13 @@ sudoers_io_log_local(int event, const char *buf, unsigned int len,
 	debug_return_int(-1);
     }
 
+    if (!log_passwords && passprompt_regex_handle != NULL) {
+	if (!iolog_pwfilt_run(passprompt_regex_handle, event, buf, len, &newbuf))
+	    debug_return_int(-1);
+    }
+
     /* Write I/O log file entry. */
-    if (iolog_write(iol, buf, len, errstr) == -1)
+    if (iolog_write(iol, newbuf ? newbuf : buf, len, errstr) == -1)
 	goto done;
 
     /* Write timing file entry. */
@@ -914,6 +977,7 @@ sudoers_io_log_local(int event, const char *buf, unsigned int len,
     ret = 1;
 
 done:
+    free(newbuf);
     debug_return_int(ret);
 }
 
